@@ -1,4 +1,4 @@
-/* $Id: code.cc 861 2008-05-25 13:23:02Z helly $ */
+/* $Id$ */
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
@@ -170,7 +170,7 @@ void Go::unmap(Go *base, const State *x)
 	nSpans = s - span;
 }
 
-void doGen(const Go *g, const State *s, uint *bm, uint f, uint m)
+static void doGen(const Go *g, const State *s, uint *bm, uint f, uint m)
 {
 	Span *b = g->span, *e = &b[g->nSpans];
 	uint lb = 0;
@@ -189,7 +189,7 @@ void doGen(const Go *g, const State *s, uint *bm, uint f, uint m)
 	}
 }
 
-void prt(std::ostream& o, const Go *g, const State *s)
+static void prt(std::ostream& o, const Go *g, const State *s)
 {
 	Span *b = g->span, *e = &b[g->nSpans];
 	uint lb = 0;
@@ -205,7 +205,7 @@ void prt(std::ostream& o, const Go *g, const State *s)
 	}
 }
 
-bool matches(const Go *g1, const State *s1, const Go *g2, const State *s2)
+static bool matches(const Go *g1, const State *s1, const Go *g2, const State *s2)
 {
 	Span *b1 = g1->span, *e1 = &b1[g1->nSpans];
 	uint lb1 = 0;
@@ -329,7 +329,7 @@ void BitMap::gen(std::ostream &o, uint ind, uint lb, uint ub)
 
 				if (yybmHexTable)
 				{
-					prtHex(o, bm[j], false);
+					prtHex(o, bm[j]);
 				}
 				else
 				{
@@ -361,7 +361,7 @@ void BitMap::stats()
 	first = NULL;
 }
 
-void genGoTo(std::ostream &o, uint ind, const State *from, const State *to, bool & readCh)
+static void genGoTo(std::ostream &o, uint ind, const State *from, const State *to, bool & readCh)
 {
 	if (DFlag)
 	{
@@ -379,7 +379,7 @@ void genGoTo(std::ostream &o, uint ind, const State *from, const State *to, bool
 	vUsedLabels.insert(to->label);
 }
 
-void genIf(std::ostream &o, uint ind, const char *cmp, uint v, bool &readCh)
+static void genIf(std::ostream &o, uint ind, const char *cmp, uint v, bool &readCh)
 {
 	o << indent(ind) << "if (";
 	if (readCh)
@@ -518,7 +518,7 @@ void Initial::emit(std::ostream &o, uint ind, bool &readCh, const std::string&)
 		o << startLabelName << ":\n";
 	}
 
-	if (vUsedLabels.count(1))
+	if (vUsedLabels.count(label+1))
 	{
 		if (state->link)
 		{
@@ -530,7 +530,7 @@ void Initial::emit(std::ostream &o, uint ind, bool &readCh, const std::string&)
 		}
 	}
 
-	if (!cFlag && vUsedLabels.count(label))
+	if (vUsedLabels.count(label))
 	{
 		o << labelPrefix << label << ":\n";
 	}
@@ -626,7 +626,7 @@ void Accept::emitBinary(std::ostream &o, uint ind, uint l, uint r, bool &readCh)
 		uint m = (l + r) >> 1;
 
 		assert(bUsedYYAccept);
-		o << indent(ind) << "if (" << mapCodeName["yyaccept"] << " <= " << m << ") {\n";
+		o << indent(ind) << "if (" << mapCodeName["yyaccept"] << (r == l+1 ? " == " : " <= ") << m << ") {\n";
 		emitBinary(o, ++ind, l, m, readCh);
 		o << indent(--ind) << "} else {\n";
 		emitBinary(o, ++ind, m + 1, r, readCh);
@@ -671,7 +671,7 @@ void Accept::emit(std::ostream &o, uint ind, bool &readCh, const std::string&) c
 				o << indent(ind) << "goto *" << mapCodeName["yytarget"] << "[" << mapCodeName["yyaccept"] << "];\n";
 				o << indent(--ind) << "}\n";
 			}
-			else if (sFlag)
+			else if (sFlag || (mapRules.size() == 2 && !DFlag))
 			{
 				emitBinary(o, ind, 0, mapRules.size() - 1, readCh);
 			}
@@ -686,11 +686,23 @@ void Accept::emit(std::ostream &o, uint ind, bool &readCh, const std::string&) c
 			else
 			{
 				o << indent(ind) << "switch (" << mapCodeName["yyaccept"] << ") {\n";
+
+				RuleMap::const_iterator it = mapRules.begin(), end = mapRules.end();
 		
-				for (RuleMap::const_iterator it = mapRules.begin(); it != mapRules.end(); ++it)
+				while (it != end)
 				{
-					o << indent(ind) << "case " << it->first << ": \t";
-					genGoTo(o, 0, state, it->second, readCh);
+					RuleMap::const_iterator tmp = it;
+
+					if (++it == end)
+					{
+						o << indent(ind) << "default:\t";
+					}
+					else
+					{
+						o << indent(ind) << "case " << tmp->first << ": \t";
+					}
+
+					genGoTo(o, 0, state, tmp->second, readCh);
 				}
 			
 				o << indent(ind) << "}\n";
@@ -713,7 +725,7 @@ void Rule::emit(std::ostream &o, uint ind, bool &, const std::string& condName)
 {
 	if (DFlag)
 	{
-		o << state->label << " [label=\"" << sourceFileInfo.fname << ":" << rule->code->line << "\"];\n";
+		o << state->label << " [label=\"" << sourceFileInfo.fname << ":" << rule->code->line << "\"]\n";
 		return;
 	}
 
@@ -750,7 +762,7 @@ void Rule::emit(std::ostream &o, uint ind, bool &, const std::string& condName)
 	o << outputFileInfo;
 }
 
-void doLinear(std::ostream &o, uint ind, Span *s, uint n, const State *from, const State *next, bool &readCh, uint mask)
+static void doLinear(std::ostream &o, uint ind, Span *s, uint n, const State *from, const State *next, bool &readCh, uint mask)
 {
 	for (;;)
 	{
@@ -830,9 +842,45 @@ void Go::genLinear(std::ostream &o, uint ind, const State *from, const State *ne
 	doLinear(o, ind, span, nSpans, from, next, readCh, mask);
 }
 
-bool genCases(std::ostream &o, uint ind, uint lb, Span *s, bool &newLine, uint mask, const State *from, const State *to)
+static void printDotCharInterval(std::ostream &o, uint lastPrintableChar, uint chr, const State *from, const State *to, bool multipleIntervals)
+{
+	o << from->label << " -> " << to->label;
+	o << " [label=";
+
+	if (lastPrintableChar != 0)
+	{
+		--chr; // we are already one char past the end
+
+		// make an interval (e.g. [A-Z])
+		if (lastPrintableChar != chr)
+		{
+			o << "\"[" << (char)lastPrintableChar << "-" << (char)chr << "]\"";
+
+			if (multipleIntervals)
+			{
+				o << "]\n";
+				o << from->label << " -> " << to->label;
+				o << " [label=";
+				prtChOrHex(o, ++chr);
+			}
+		}
+		else
+		{
+			prtChOrHex(o, chr);
+		}
+	}
+	else
+	{
+		prtChOrHex(o, chr);
+	}
+
+	o << "]";
+}
+
+static bool genCases(std::ostream &o, uint ind, uint lb, Span *s, bool &newLine, uint mask, const State *from, const State *to)
 {
 	bool used = false;
+	uint lastPrintableChar = 0;
 
 	if (!newLine)
 	{
@@ -847,16 +895,32 @@ bool genCases(std::ostream &o, uint ind, uint lb, Span *s, bool &newLine, uint m
 			{
 				if (DFlag)
 				{
-					o << from->label << " -> " << to->label;
-					o << " [label=";
-					prtChOrHex(o, lb);
-					o << "]";
+					if ((lb >= 'A' && lb <= 'Z') || (lb >= 'a' && lb <= 'z') || (lb >= '0' && lb <= '9'))
+					{
+						if (lastPrintableChar == 0)
+						{
+							lastPrintableChar = lb;
+						}
+
+						if (++lb == s->ub)
+						{
+							break;
+						}
+						continue;
+					}
+
+					printDotCharInterval(o, lastPrintableChar, lb, from, to, true);
+					lastPrintableChar = 0;
 				}
 				else
 				{
 					o << indent(ind) << "case ";
 					prtChOrHex(o, lb);
 					o << ":";
+					if (dFlag && eFlag && lb < 256u && isprint(talx[lb]))
+					{
+						o << " /* " << std::string(1, talx[lb]) << " */";
+					}
 				}
 				newLine = false;
 				used = true;
@@ -871,6 +935,15 @@ bool genCases(std::ostream &o, uint ind, uint lb, Span *s, bool &newLine, uint m
 			newLine = true;
 		}
 	}
+
+	if (lastPrintableChar != 0)
+	{
+		printDotCharInterval(o, lastPrintableChar, lb, from, to, false);
+
+		o << "\n";
+		newLine = true;
+	}
+
 	return used;
 }
 
@@ -954,7 +1027,13 @@ void Go::genSwitch(std::ostream &o, uint ind, const State *from, const State *ne
 
 		if (DFlag)
 		{
-			o << "\n" << from->label << " -> " << def->label;
+			if (!newLine)
+			{
+				o << "\n";
+				newLine = true;
+			}
+
+			o << from->label << " -> " << def->label;
 			o << " [label=default]\n" ;
 		}
 		else
@@ -968,7 +1047,7 @@ void Go::genSwitch(std::ostream &o, uint ind, const State *from, const State *ne
 	}
 }
 
-void doBinary(std::ostream &o, uint ind, Span *s, uint n, const State *from, const State *next, bool &readCh, uint mask)
+static void doBinary(std::ostream &o, uint ind, Span *s, uint n, const State *from, const State *next, bool &readCh, uint mask)
 {
 	if (n <= 4)
 	{
@@ -1208,7 +1287,7 @@ void Go::genGoto(std::ostream &o, uint ind, const State *from, const State *next
 					o << "if (" << mapCodeName["yybm"] << "[" << b->i << "+" << sYych << "] & ";
 					if (yybmHexTable)
 					{
-						prtHex(o, b->m, false);
+						prtHex(o, b->m);
 					}
 					else
 					{
@@ -1245,7 +1324,7 @@ void State::emit(std::ostream &o, uint ind, bool &readCh, const std::string& con
 	action->emit(o, ind, readCh, condName);
 }
 
-uint merge(Span *x0, State *fg, State *bg)
+static uint merge(Span *x0, State *fg, State *bg)
 {
 	Span *x = x0, *f = fg->go.span, *b = bg->go.span;
 	uint nf = fg->go.nSpans, nb = bg->go.nSpans;
@@ -1320,7 +1399,7 @@ uint merge(Span *x0, State *fg, State *bg)
 	}
 }
 
-const uint cInfinity = ~0u;
+static const uint cInfinity = ~0u;
 
 class SCC
 {
@@ -1421,7 +1500,7 @@ static bool state_is_in_non_trivial_SCC(const State* s)
 	return false;
 }
 
-uint maxDist(State *s)
+static uint maxDist(State *s)
 {
 	if (s->depth != cInfinity)
 	{
@@ -1458,7 +1537,7 @@ uint maxDist(State *s)
 	return mm;
 }
 
-void calcDepth(State *head)
+static void calcDepth(State *head)
 {
 	State* s;
 
@@ -1704,6 +1783,18 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 		bUsedYYAccept = false;
 	}
 	
+	// In -c mode, the prolog needs its own label separate from start_label.
+	// prolog_label is before the condition branch (GenCondGoto). It is
+	// equivalent to startLabelName.
+	// start_label corresponds to current condition.
+	// NOTE: prolog_label must be yy0 because of the !getstate:re2c handling
+	// in scanner.re
+	uint prolog_label = next_label;
+	if (bProlog && cFlag)
+	{
+		next_label++;
+	}
+
 	uint start_label = next_label;
 
 	(void) new Initial(head, next_label++, bSaveOnHead);
@@ -1712,7 +1803,7 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 	{
 		if (startLabelName.empty())
 		{
-			vUsedLabels.insert(start_label);
+			vUsedLabels.insert(prolog_label);
 		}
 	}
 
@@ -1788,12 +1879,12 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 	if (bProlog)
 	{
 		genCondTable(o, ind, *specMap);
-		genGetStateGoto(o, ind, start_label);
-		if (cFlag)
+		genGetStateGoto(o, ind, prolog_label);
+		if (cFlag && !DFlag)
 		{
-			if (vUsedLabels.count(start_label))
+			if (vUsedLabels.count(prolog_label))
 			{
-				o << labelPrefix << start_label << ":\n";
+				o << labelPrefix << prolog_label << ":\n";
 			}
 			if (!startLabelName.empty())
 			{
@@ -1825,11 +1916,11 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 		BitMap::gen(o, ind, lbChar, ubChar <= 256 ? ubChar : 256);
 	}
 
-	// TODO: Shouldn't labels 0 and 1 be variable?
-	if (vUsedLabels.count(1))
+	// The start_label is not always the first to be emitted, so we may have to jump. c.f. Initial::emit()
+	if (vUsedLabels.count(start_label+1))
 	{
-		vUsedLabels.insert(0);
-		o << indent(ind) << "goto " << labelPrefix << "0;\n";
+		vUsedLabels.insert(start_label);
+		o << indent(ind) << "goto " << labelPrefix << start_label << ";\n";
 	}
 
 	// Generate code
@@ -1860,7 +1951,7 @@ void DFA::emit(std::ostream &o, uint& ind, const RegExpMap* specMap, const std::
 	bUseStartLabel = false;
 }
 
-void genGetStateGotoSub(std::ostream &o, uint ind, uint start_label, int cMin, int cMax)
+static void genGetStateGotoSub(std::ostream &o, uint ind, uint start_label, int cMin, int cMax)
 {
 	if (cMin == cMax)
 	{
