@@ -1,4 +1,4 @@
-/* $Id: main.cc 858 2008-04-03 20:53:44Z helly $ */
+/* $Id$ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #elif defined(_WIN32)
@@ -368,9 +368,9 @@ int main(int argc, char *argv[])
 		return 2;
 	}
 
-	if (dFlag && DFlag)
+	if (DFlag && (bFlag || dFlag || sFlag))
 	{
-		std::cerr << "re2c: error: Cannot combine -d with -D switch\n";
+		std::cerr << "re2c: error: Cannot combine -D with -b, -d  or -s switches\n";
 		return 2;
 	}
 
